/*setup includes*/
#include <windows.h>
#include <tchar.h>
#include "modulebox.h"
#include "../../lssrc/ls-b24/lsapi/lsapi.h"

/*setup constants*/
const char szAppName[] = _T("ModuleBox");
const char szAppClass[] = _T("ModuleBoxClass");
const char rcsRevision[] = _T("$Revision: 0.30 $");
const char rcsId[] = _T("$Id: modulebox.cpp,v 0.30 2000/01/11 00:22:00 jugg $");

/*setup variables*/
HWND parentWnd, mainWnd, desktopWnd;
int screenHeight, screenWidth, WndX, WndY;
ModBoxSettings mbs;

/*function declarations*/
LRESULT CALLBACK WndProc(HWND,UINT,WPARAM,LPARAM);
void ModBoxHide(void);
void ModBoxMove(HWND,LPSTR);
void ModBoxShow(void);
void ModBoxToggle(void);
void ModBoxzOrder(HWND,LPSTR);


ModBoxSettings ReadSettings(void)
{
	ModBoxSettings settings;
	TCHAR tmp[MAX_PATH];

	settings.Hidden = GetRCBool(_T("ModBoxHidden"), TRUE);
		
	settings.BgColor = GetRCColor(_T("ModBoxBgColor"), 0xFFFFFF);
	settings.BorderColor = GetRCColor(_T("ModBoxBorderColor"), 0x000000);

	settings.Height = GetRCInt(_T("ModBoxHeight"), 32);
		if (settings.Height < 2) settings.Height = 32;
	settings.Width = GetRCInt(_T("ModBoxWidth"), 32);
		if (settings.Width < 2) settings.Width = 32;

	settings.X = GetRCInt(_T("ModBoxX"), 0);
		if (settings.X < 0)
			WndX = screenWidth + settings.X;
		else
			WndX = settings.X;
	settings.Y = GetRCInt(_T("ModBoxY"), 0);
		if (settings.Y < 0)
			WndY = screenHeight + settings.Y;
		else
			WndY = settings.Y;

	settings.BorderSize = GetRCInt(_T("ModBoxBorderSize"), 2);
		if (settings.BorderSize < 0) settings.BorderSize = 2;

	GetRCString(_T("ModBoxBgImage"), tmp, _T(".none"), MAX_PATH);
		settings.BgImage = LoadLSImage(tmp, NULL);

	settings.zOrder = GetRCInt(_T("ModBoxzOrder"), 0);
		if (settings.zOrder < 0 || settings.zOrder > 2) settings.zOrder = 0;

	return settings;
}


int initModuleEx(HWND parent, HINSTANCE hInst, LPCSTR szPath)
{
	UINT Msgs[2];
	szPath=szPath;
	parentWnd = parent;

	screenHeight = GetSystemMetrics(SM_CYSCREEN);
	screenWidth = GetSystemMetrics(SM_CXSCREEN);

	desktopWnd = NULL;
	desktopWnd = FindWindow(_T("DesktopBackgroundClass"), NULL);
	if (!desktopWnd)
		desktopWnd = GetDesktopWindow();

	{
		WNDCLASS wc;
		memset(&wc,0,sizeof(wc));
		wc.cbClsExtra = NULL;
		wc.cbWndExtra = NULL;
		wc.hbrBackground = NULL;
		wc.hCursor = LoadCursor(NULL, IDC_ARROW);
		wc.hIcon = NULL;
		wc.hInstance = hInst;
		wc.lpfnWndProc = WndProc;
		wc.lpszClassName = szAppClass;
		wc.lpszMenuName = NULL;
		wc.style = NULL;
		if (!RegisterClass(&wc))
		{
			MessageBox(parentWnd,_T("Error registering window class"),_T("ModuleBox"),MB_OK|MB_ICONERROR|MB_DEFBUTTON1|MB_SYSTEMMODAL);
			return 1;
		}
	}

	mbs = ReadSettings();

	mainWnd = CreateWindowEx(WS_EX_TOOLWINDOW|WS_EX_ACCEPTFILES|(mbs.zOrder==2 ? WS_EX_TOPMOST:0), szAppClass, szAppName, WS_POPUP, WndX,WndY,mbs.Width,mbs.Height, NULL, NULL, hInst, NULL);
	if (!mainWnd){
		MessageBox(parentWnd,_T("Error creating window"),_T("ModuleBox"),MB_OK|MB_ICONERROR|MB_DEFBUTTON1|MB_SYSTEMMODAL);
		return 1;
	}
	if (!mbs.zOrder)
		SetParent(mainWnd,desktopWnd);
	SetWindowLong(mainWnd, GWL_USERDATA, 0x49474541);

	{// register with LiteStep to receive Version request, so we display in the !about box.
		Msgs[0] = LM_GETREVID;
		Msgs[1] = 0;
		SendMessage(parentWnd, LM_REGISTERMESSAGE, (WPARAM) mainWnd, (LPARAM) Msgs);
	}

	AddBangCommand(_T("!MODBOXHIDE"), ModBoxHide);
	AddBangCommand(_T("!MODBOXMOVE"), ModBoxMove);
	AddBangCommand(_T("!MODBOXSHOW"), ModBoxShow);
	AddBangCommand(_T("!MODBOXTOGGLE"), ModBoxToggle);
	AddBangCommand(_T("!MODBOXZORDER"), ModBoxzOrder);

	if (mbs.Hidden)
		ShowWindow(mainWnd, SW_HIDE);
	else
		ShowWindow(mainWnd, SW_SHOWNOACTIVATE);

	return 0;
}

void quitModule(HINSTANCE hInst)
{
	UINT Msgs[2];
	{// unregister the litestep version info message.
		Msgs[0] = LM_GETREVID;
		Msgs[1] = 0;
		SendMessage(parentWnd, LM_UNREGISTERMESSAGE, (WPARAM) mainWnd, (LPARAM) Msgs);
	}
	RemoveBangCommand(_T("!MODBOXHIDE"));
	RemoveBangCommand(_T("!MODBOXMOVE"));
	RemoveBangCommand(_T("!MODBOXSHOW"));
	RemoveBangCommand(_T("!MODBOXTOGGLE"));
	RemoveBangCommand(_T("!MODBOXZORDER"));

	DestroyWindow(mainWnd);
	UnregisterClass(szAppClass,hInst);
	return;
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case LM_GETREVID:
		{
			LPSTR buf = (LPSTR)lParam;
			if (!wParam){
				strcpy(buf, _T("modulebox.dll: "));
				strcat(buf, &rcsRevision[11]);
				buf[strlen(buf)-1] = _T('\0');
			}
			else if (wParam == 1){
				strcpy(buf, &rcsId[1]);
				buf[strlen(buf)-1] = _T('\0');
			}else{
				strcpy(buf, _T(""));
			}
			return strlen(buf);
		}
		case WM_DROPFILES:
		{
			TCHAR szFileName[MAX_PATH];
			int numFiles;
			UINT keyState = GetAsyncKeyState(VK_SHIFT) & 0x8000;
			numFiles = DragQueryFile((HDROP)wParam, 0xFFFFFFFF, NULL, NULL);
			for (int i=0;i<numFiles;i++){
				DragQueryFile((HDROP)wParam, i, szFileName, sizeof(szFileName));
				if (GetFileAttributes(szFileName) != 0xFFFFFFFF){
					if (keyState)
						ParseBangCommand(hwnd, _T("!UnLoadModule"), szFileName);
					else
						ParseBangCommand(hwnd, _T("!ReLoadModule"), szFileName);
				}
			}
			DragFinish((HDROP)wParam);
			return 0;
		}
		case WM_ENDSESSION:
		case WM_QUERYENDSESSION:
			return SendMessage(parentWnd,message,wParam,lParam);
		case WM_PAINT:
		{
			HDC hdc;//, hdcBuf;
			//HBITMAP hOldBmp, hFinalBmp;
			PAINTSTRUCT ps;
			//BITMAP bmInfo;
			HBRUSH hBG, hOldBg;

			RECT r;
			GetClientRect(hwnd, &r);

			//if (mbs.BgImage){
			//	GetObject(mbs.BgImage, sizeof(bmInfo), &bmInfo);
			//}

			if (!wParam)
				hdc = BeginPaint(hwnd,&ps);
			else
				hdc = (HDC) wParam;

			hBG = CreateSolidBrush(mbs.BgColor);
			hOldBg = (HBRUSH) SelectObject(hdc, hBG);
			FillRect(hdc, &r, hBG);
			SelectObject(hdc, hOldBg);
			DeleteObject(hBG);

			if (mbs.BorderSize > 0){
				HPEN pen, oldpen;
				pen = CreatePen(PS_SOLID|PS_INSIDEFRAME, mbs.BorderSize*2, mbs.BorderColor);
				oldpen = (HPEN) SelectObject(hdc, pen);
				MoveToEx(hdc, r.left, r.bottom, NULL);
				LineTo(hdc, r.left, r.top);
				LineTo(hdc, r.right, r.top);
				LineTo(hdc, r.right, r.bottom);
				LineTo(hdc, r.left, r.bottom);
				SelectObject(hdc, oldpen);
				DeleteObject(pen);
			}

			//hdcBuf = CreateCompatibleDC(hdc);

			//if (mbs.BgImage && mbs.BgColor == 0x00FF00FF){
			//	hFinalBmp = CreateCompatibleBitmap(hdc, mbs.Width, mbs.Height);
			//	hOldBmp = (HBITMAP) SelectObject(hdcBuf, hFinalBmp);
			//}

			if (mbs.BgImage){
				BITMAP bmInfo;
				GetObject(mbs.BgImage, sizeof(bmInfo), &bmInfo);
				HDC src = CreateCompatibleDC(hdc);
				HBITMAP oldBmp = (HBITMAP) SelectObject(src, mbs.BgImage);
				TransparentBltLS(hdc, (mbs.Width/2 - bmInfo.bmWidth/2), (mbs.Height/2 - bmInfo.bmHeight/2), bmInfo.bmWidth, bmInfo.bmHeight, src, 0, 0, RGB(255, 0, 255));
				SelectObject(src, oldBmp);
				DeleteDC(src);
			}

			//if (mbs.BgColor != 0x00FF00FF && mbs.BorderSize > 0){
				//Code for painting a rectangle, using the API function.
			//}

			//if (mbs.BgImage && mbs.BgColor == 0x00FF00FF){
			//	SelectObject(hdcBuf,hOldBmp);
			//	HRGN tile = BitmapToRegion(hFinalBmp, RGB(255,0,255), 0x101010, 0, 0);
			//	SetWindowRgn(hwnd,tile,true);
			//	hOldBmp = (HBITMAP) SelectObject(hdcBuf,hFinalBmp);
			//}

			//BitBlt(hdc,0,0,mbs.Width,mbs.Height,hdcBuf,0,0,SRCCOPY);
			//if (mbs.BgImage && mbs.BgColor == 0x00FF00FF)
			//	SelectObject(hdcBuf,hOldBmp);
			//DeleteDC(hdcBuf);
			if (!wParam)
					EndPaint(hwnd,&ps);
			return 0;
		}
		case WM_DISPLAYCHANGE:
		{
				screenHeight = HIWORD(lParam);
				screenWidth = LOWORD(lParam);
				if (mbs.X < 0)
					WndX = screenWidth + mbs.X;
				else
					WndX = mbs.X;
				if (mbs.Y < 0)
					WndY = screenHeight + mbs.Y;
				else
					WndY = mbs.Y;
				SetWindowPos(hwnd,0,WndX,WndY,0,0,SWP_NOSIZE|SWP_NOZORDER);
				// Use MoveWindow instead
				break;
		}
		case WM_CLOSE:
		case WM_SYSCOMMAND:
			return 0;
	}
	return DefWindowProc(hwnd,message,wParam,lParam);
}

void ModBoxHide(void)
{
	if (!mbs.Hidden){
		ShowWindow(mainWnd, SW_HIDE);
		mbs.Hidden = TRUE;
	}
	return;
}

void ModBoxMove(HWND hwnd, LPSTR args)
{
	hwnd = hwnd;
	POINT ps;
	GetCursorPos(&ps);
	WndX = ps.x;
	WndY = ps.y;
	if (args && strlen(args)){
		if (!lstrcmpi(_T("home"), args)){
			if (mbs.X < 0)
				WndX = screenWidth + mbs.X;
			else
				WndX = mbs.X;
			if (mbs.Y < 0)
				WndY = screenHeight + mbs.Y;
			else
				WndY = mbs.Y;
		}else{
			args = strtok(args, "\"\t ,");
				if (args != NULL){
					WndX = atoi(args);
					if (WndX < 0) WndX = screenWidth + WndX;
				}
			args = strtok(NULL, "\"\t ,");
				if (args != NULL){
					WndY = atoi(args);
					if (WndY < 0) WndY = screenHeight + WndY;
				}
		}
	}
	SetWindowPos(mainWnd,0,WndX,WndY,0,0,SWP_NOSIZE|SWP_NOZORDER);
	return;
}

void ModBoxShow(void)
{
	if (mbs.Hidden){
		ShowWindow(mainWnd, SW_SHOWNOACTIVATE);
		mbs.Hidden = FALSE;
	}
	return;
}

void ModBoxToggle(void)
{
	if (mbs.Hidden)
		ShowWindow(mainWnd, SW_SHOWNOACTIVATE);
	else
		ShowWindow(mainWnd, SW_HIDE);
	mbs.Hidden = !mbs.Hidden;
	return;
}

void ModBoxzOrder(HWND hwnd, LPSTR args)
{
	hwnd=hwnd;
	if (args && strlen(args)){
		if (atoi(args) == 0){
			SetWindowPos(hwnd,HWND_BOTTOM,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE|SWP_NOACTIVATE);
			SetParent(hwnd, desktopWnd);
		}
		else if (atoi(args) == 2){
			SetWindowPos(hwnd,HWND_TOPMOST,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE|SWP_NOACTIVATE);
			SetParent(hwnd, NULL);
		}else{
			SetWindowPos(hwnd,HWND_NOTOPMOST,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE|SWP_NOACTIVATE);
			SetParent(hwnd, NULL);
		}
	}
	return;
}
